/*
    Rosegarden
    A MIDI and audio sequencer and musical notation editor.

    This program is Copyright 2000-2008
        Guillaume Laurent   <glaurent@telegraph-road.org>,
        Chris Cannam        <cannam@all-day-breakfast.com>,
        Richard Bown        <richard.bown@ferventsoftware.com>

    The moral rights of Guillaume Laurent, Chris Cannam, and Richard
    Bown to claim authorship of this work have been asserted.

    Other copyrights also apply to some parts of this work.  Please
    see the AUTHORS file and individual file headers for details.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/

#ifndef _RG_MIDICONFIGURATIONPAGE_H_
#define _RG_MIDICONFIGURATIONPAGE_H_

#include "TabbedConfigurationPage.h"
#include <tqstring.h>
#include <tdelocale.h>
#include <tqlineedit.h>
#include <tqcheckbox.h>


class TQWidget;
class TQSpinBox;
class TQSlider;
class TQPushButton;
class TQLabel;
class TQComboBox;
class TDEConfig;
class KComboBox;


namespace Rosegarden
{

class RosegardenGUIDoc;


class MIDIConfigurationPage : public TabbedConfigurationPage
{
    TQ_OBJECT
  
public:
    MIDIConfigurationPage(RosegardenGUIDoc *doc,
                               TDEConfig *cfg,
                               TQWidget *parent=0,
                               const char *name=0);

    virtual void apply();

    static TQString iconLabel() { return i18n("MIDI"); }
    static TQString title()     { return i18n("MIDI Settings"); }
    static TQString iconName()  { return "configure-midi"; }

protected slots:
    void slotSoundFontToggled(bool);
    void slotSfxLoadPathChoose();
    void slotSoundFontChoose();

protected:
    bool getUseDefaultStudio()      { return m_studio->isChecked(); }

    //--------------- Data members ---------------------------------

    // General
    TQCheckBox *m_sendControllersAtPlay;

    TQCheckBox   *m_sfxLoadEnabled;
    TQLineEdit   *m_sfxLoadPath;
    TQPushButton *m_sfxLoadChoose;
    TQLineEdit   *m_soundFontPath;
    TQPushButton *m_soundFontChoose;

    // Sync and timing
    //
    //TQCheckBox *m_midiClockEnabled;
    TQComboBox *m_midiSync;
    TQString    m_origTimer;
    TQComboBox *m_timer;
    TQComboBox *m_mmcTransport;
    TQComboBox *m_mtcTransport;
    TQCheckBox *m_midiSyncAuto;

    TQCheckBox* m_studio;
    TQSpinBox*  m_midiPitchOctave;

};
 


}

#endif
