/*
    Rosegarden
    A sequencer and musical notation editor.
 
    This program is Copyright 2000-2008
        Guillaume Laurent   <glaurent@telegraph-road.org>,
        Chris Cannam        <cannam@all-day-breakfast.com>,
        Richard Bown        <bownie@bownie.com>
 
    The moral right of the authors to claim authorship of this work
    has been asserted.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/

#include <tqapplication.h>
#include <tqeventloop.h>

#include <tdemainwindow.h>
#include <kstatusbar.h>
#include <tdelocale.h>

#include "KTmpStatusMsg.h"
#include "gui/application/RosegardenApplication.h"

KTmpStatusMsg::KTmpStatusMsg(const TQString& msg, TDEMainWindow* window, int id)
        : m_mainWindow(window),
        m_id(id)
{
    m_mainWindow->statusBar()->changeItem(TQString("  %1").arg(msg), m_id);
    Rosegarden::rgapp->refreshGUI(50);
}

KTmpStatusMsg::~KTmpStatusMsg()
{
    m_mainWindow->statusBar()->clear();
    m_mainWindow->statusBar()->changeItem(m_defaultMsg, m_id);
    Rosegarden::rgapp->refreshGUI(50);
}


void KTmpStatusMsg::setDefaultMsg(const TQString& m)
{
    m_defaultMsg = m;
}

const TQString& KTmpStatusMsg::getDefaultMsg()
{
    return m_defaultMsg;
}

void KTmpStatusMsg::setDefaultId(int id)
{
    m_defaultId = id;
}

int KTmpStatusMsg::getDefaultId()
{
    return m_defaultId;
}


int KTmpStatusMsg::m_defaultId = 1;
TQString KTmpStatusMsg::m_defaultMsg = "";
