<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="Rosegarden::TempoView" version="1500">

<MenuBar>

<Menu name="edit" append="new_merge">
  <Action name="insert_tempo"/>
  <Action name="insert_timesig"/>
  <Action name="delete"/>
  <Action name="edit"/>
  <Separator/>
  <Action name="select_all"/>
  <Action name="clear_selection"/>
</Menu>

<Menu name="view"><text>&amp;View</text>
  <Action name="time_musical"/>
  <Action name="time_real"/>
  <Action name="time_raw"/>
</Menu>

</MenuBar>

<ToolBar name="Actions Toolbar">
  <text>Actions Toolbar</text>
  <Action name="insert_tempo"/>
  <Action name="insert_timesig"/>
  <Action name="delete"/>
  <Action name="edit"/>
</ToolBar>

<ToolBar name="Time Toolbar">
  <text>Time Toolbar</text>
  <Action name="time_musical"/>
  <Action name="time_real"/>
  <Action name="time_raw"/>
</ToolBar>

</kpartgui>

<ToolBar name="Transport Toolbar">
  <text>Transport Toolbar</text>

  <Action name="playback_pointer_start" />
  <Action name="playback_pointer_back_bar" />
  <Action name="play" />
  <Action name="playback_pointer_forward_bar" />
  <Action name="playback_pointer_end" />
  <Action name="stop" />
  <Action name="playback_pointer_to_cursor" />
  <Action name="cursor_to_playback_pointer" />
  <Action name="toggle_solo"/>
  <Action name="panic"/>
</ToolBar>

<ToolBar name="Actions Toolbar">
  <text>Actions Toolbar</text>
</ToolBar>

<ToolBar name="Zoom Toolbar">
  <text>Zoom Toolbar</text>
</ToolBar>

<State name="have_selection">
  <enable>
    <Action name="edit_cut"/>
    <Action name="edit_copy"/>
    <Action name="edit"/>
    <Action name="quantize"/>
    <Action name="velocity_up"/>
    <Action name="velocity_down"/>
    <Action name="set_velocities"/>
    <Action name="transpose_up"/>
    <Action name="transpose_down"/>
    <Action name="transpose_up_octave"/>
    <Action name="transpose_down_octave"/>
    <Action name="general_transpose"/>
    <Action name="general_diatonic_transpose"/>
    <Action name="invert"/>
    <Action name="retrograde"/>
    <Action name="retrograde_invert"/>
    <Action name="preview_selection"/>
    <Action name="clear_selection"/>
    <Action name="filter_selection"/>
  </enable>
</State>

<State name="have_control_ruler">
  <enable>
    <Action name="show_control_rulers"/>
  </enable>
</State>

</kpartgui>
 
