/*
    Rosegarden
    A sequencer and musical notation editor.

    This program is Copyright 2000-2008
        Guillaume Laurent   <glaurent@telegraph-road.org>,
        Chris Cannam        <cannam@all-day-breakfast.com>,
        Richard Bown        <bownie@bownie.com>

    The moral right of the authors to claim authorship of this work
    has been asserted.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/

/**
 * Miscellaneous constants
 */


#ifndef _CONSTANTS_H_
#define _CONSTANTS_H_

namespace Rosegarden 
{
    extern const char* const GeneralOptionsConfigGroup;
    extern const char* const LatencyOptionsConfigGroup;
    extern const char* const SequencerOptionsConfigGroup;
    extern const char* const NotationViewConfigGroup;
    extern const char* const AudioManagerDialogConfigGroup;
    extern const char* const SynthPluginManagerConfigGroup;
    extern const char* const BankEditorConfigGroup;
    extern const char* const ColoursConfigGroup;
    extern const char* const ControlEditorConfigGroup;
    extern const char* const DeviceManagerConfigGroup;
    extern const char* const EventFilterDialogConfigGroup;
    extern const char* const EventViewLayoutConfigGroupName;
    extern const char* const EventViewConfigGroup;
    extern const char* const MarkerEditorConfigGroup;
    extern const char* const MatrixViewConfigGroup;
    extern const char* const PlayListConfigGroup;
    extern const char* const MainWindowConfigGroup;
    extern const char* const TransportDialogConfigGroup;
    extern const char* const TempoViewLayoutConfigGroupName;
    extern const char* const TempoViewConfigGroup;
    extern const char* const TriggerManagerConfigGroup;
    extern const char* const EditViewConfigGroup;
}

#endif
