<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="Rosegarden::AudioManagerDialog" version="1230">
  
<MenuBar>

<Menu name="file">
    <Action name="add_audio" append="new_merge"/>
    <Action name="export_audio" append="new_merge"/>
</Menu>

<Menu name="edit">
    <Action name="remove_audio"/>
    <Action name="remove_all_audio"/>
    <Action name="remove_all_unused_audio"/>
    <Separator/>
    <Action name="delete_unused_audio"/>
</Menu>

<Menu name="action">
<Text>&amp;Action</Text>
    <Action name="preview_audio"/>
    <Action name="insert_audio"/>
    <Action name="distribute_audio"/>
</Menu>
</MenuBar>

<ToolBar name="General Toolbar">
  <text>General Toolbar</text>

  <Action name="add_audio" />
  <Action name="export_audio" />
  <Action name="remove_audio" />
  <Action name="preview_audio" />
  <Action name="insert_audio" />
</ToolBar>

<State name="have_audio_files">
  <enable>
    <Action name="remove_all_audio"/>
    <Action name="remove_all_unused_audio"/>
    <Action name="delete_unused_audio"/>
  </enable>
</State>

<State name="have_audio_selected">
  <enable>
    <Action name="export_audio"/>
    <Action name="remove_audio"/>
    <Action name="rename_audio"/>
    <Action name="distribute_audio"/>
  </enable>
</State>

<State name="have_audible_preview">
  <enable>
    <Action name="preview_audio"/>
  </enable>
</State>

<State name="have_audio_insertable">
  <enable>
    <Action name="insert_audio"/>
  <enable>
</State>

</kpartgui>
