/*
    Rosegarden
    A MIDI and audio sequencer and musical notation editor.

    This program is Copyright 2000-2008
        Guillaume Laurent   <glaurent@telegraph-road.org>,
        Chris Cannam        <cannam@all-day-breakfast.com>,
        Richard Bown        <richard.bown@ferventsoftware.com>

    The moral rights of Guillaume Laurent, Chris Cannam, and Richard
    Bown to claim authorship of this work have been asserted.

    Other copyrights also apply to some parts of this work.  Please
    see the AUTHORS file and individual file headers for details.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/

#ifndef _RG_IMPORTDEVICEDIALOG_H_
#define _RG_IMPORTDEVICEDIALOG_H_

#include "base/MidiDevice.h"
#include <string>
#include <kdialogbase.h>
#include <tqstring.h>
#include <vector>
#include <kurl.h>


class TQWidget;
class TQRadioButton;
class TQLabel;
class TQCheckBox;
class TQButtonGroup;
class ProgramList;
class KeyMappingList;
class KComboBox;
class ControlList;
class BankList;


namespace Rosegarden
{

class RosegardenGUIDoc;


class ImportDeviceDialog : public KDialogBase
{
    TQ_OBJECT
  

public:
    ImportDeviceDialog(TQWidget *parent, KURL url);
    virtual ~ImportDeviceDialog();

    bool doImport();

    bool shouldImportBanks() const;
    bool shouldImportKeyMappings() const;
    bool shouldImportControllers() const;
    bool shouldOverwriteBanks() const; 
    bool shouldRename() const;

    std::string getDeviceName() const;
    const BankList &getBanks() const;
    const ProgramList &getPrograms() const;
    const KeyMappingList &getKeyMappings() const;
    const ControlList &getControllers() const;
    std::string getLibrarianName() const;
    std::string getLibrarianEmail() const;
    MidiDevice::VariationType getVariationType() const;

public slots:
    void slotOk();
    void slotCancel();

protected:
    bool importFromRG(TQString fileName);
    bool importFromSF2(TQString fileName);

    KURL               m_url;

    KComboBox          *m_deviceCombo;
    TQLabel             *m_deviceLabel;

    TQCheckBox          *m_importBanks;
    TQCheckBox          *m_importKeyMappings;
    TQCheckBox          *m_importControllers;
    TQCheckBox          *m_rename;

    TQButtonGroup       *m_buttonGroup;
    TQRadioButton       *m_mergeBanks;
    TQRadioButton       *m_overwriteBanks;

    RosegardenGUIDoc   *m_fileDoc;
    std::vector<MidiDevice *> m_devices;
    MidiDevice *m_device;
};


}

#endif
