<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="Rosegarden::NotationTool" version="1230">

<Menu noMerge="1" name="NoteInserter">
  <Action name="select" />
  <Action name="erase" />
  <Action name="rests" />
  <Separator/>
  <Action name="toggle_auto_beam" />
  <Separator/>
  <Action name="toggle_dot" />
  <Separator/>
  <Action name="no_accidental" />
  <Action name="follow_accidental" />
  <Action name="sharp_accidental" />
  <Action name="flat_accidental" />
  <Action name="natural_accidental" />
  <Action name="double_sharp_accidental" />
  <Action name="double_flat_accidental" />
</Menu>

</kpartgui>
