/*
    Rosegarden
    A MIDI and audio sequencer and musical notation editor.

    This program is Copyright 2000-2008
        Guillaume Laurent   <glaurent@telegraph-road.org>,
        Chris Cannam        <cannam@all-day-breakfast.com>,
        Richard Bown        <richard.bown@ferventsoftware.com>

    The moral rights of Guillaume Laurent, Chris Cannam, and Richard
    Bown to claim authorship of this work have been asserted.

    Other copyrights also apply to some parts of this work.  Please
    see the AUTHORS file and individual file headers for details.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/

#ifndef _RG_PROPERTYVIEWRULER_H_
#define _RG_PROPERTYVIEWRULER_H_

#include "base/PropertyName.h"
#include "gui/general/HZoomable.h"
#include <tqfont.h>
#include <tqfontmetrics.h>
#include <tqsize.h>
#include <tqwidget.h>


class TQPaintEvent;


namespace Rosegarden
{

class Segment;
class RulerScale;


/**
 * PropertyViewRuler is a widget that shows a range of Property
 * (velocity, typically) values for a set of Rosegarden Events.
 */
class PropertyViewRuler : public TQWidget, public HZoomable
{
    TQ_OBJECT
  

public:
    PropertyViewRuler(RulerScale *rulerScale,
                      Segment *segment,
                      const PropertyName &property,
                      double xorigin = 0.0,
                      int height = 0,
                      TQWidget* parent = 0,
                      const char *name = 0);

    ~PropertyViewRuler();

    virtual TQSize sizeHint() const;
    virtual TQSize minimumSizeHint() const;

    void setMinimumWidth(int width) { m_width = width; }

    /**
     * Get the property name
     */
    PropertyName getPropertyName() const { return m_propertyName; }

public slots:
    void slotScrollHoriz(int x);

protected:
    virtual void paintEvent(TQPaintEvent *);

    //--------------- Data members ---------------------------------

    PropertyName m_propertyName;

    double m_xorigin;
    int    m_height;
    int    m_currentXOffset;
    int    m_width;

    Segment     *m_segment;
    RulerScale  *m_rulerScale;

    TQFont                    m_font;
    TQFont                    m_boldFont;
    TQFontMetrics             m_fontMetrics;
};


}

#endif
