<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="BankEditor" version="1230">

<MenuBar>
  <Menu name="file">
   <Action name="file_close_discard"/>
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar">
   <Action name="file_close"/>
</ToolBar>

<State name="on_bank_item">
     <enable>
        <Action name="edit_copy"/>
        <Action name="edit_paste"/>
     </enable>
</State>

</kpartgui>
