<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="Rosegarden::Notation" version="1500">

<MenuBar>

<Menu name="file" append="new_merge">
<Action name="file_print_lilypond" append="print_merge" />
<Action name="file_preview_lilypond" append="print_merge" />
</Menu>

<Menu name="edit">
  <Action name="cut_and_close"/>
  <Action name="general_paste"/>
  <Action name="delete"/>
  <Separator/>
    <Action name="move_events_up_staff"/>
    <Action name="move_events_down_staff"/>
  <Separator/>
  <Action name="select_from_start"/>
  <Action name="select_to_end"/>
  <Action name="select_whole_staff"/>
  <Action name="clear_selection"/>
  <Action name="filter_selection"/>
</Menu>

<Menu name="View">
  <text>&amp;View</text>
<!--
<Action name="debug_dump"/>
<Separator/>
-->
  <Action name="note_font_actionmenu"/>
  <Action name="note_font_size_actionmenu"/>
  <Action name="stretch_actionmenu"/>
  <Action name="proportion_actionmenu"/>
<Separator/>
  <Menu name="layout"><text>&amp;Layout Mode</text>
    <Action name="linear_mode"/>
    <Action name="continuous_page_mode"/>
    <Action name="multi_page_mode"/>
  </Menu>
<Separator/>
  <Action name="lyric_editor"/>
  <Separator/>
    <Action name="show_velocity_control_ruler"/>
    <Action name="show_controller_events_ruler"/>
<!--    <Action name="add_control_ruler"/> -->
    <Menu name="add_control_ruler"><text>Add Event Ruler</text></Menu>
    <Separator/>
    <Action name="show_track_headers"/>
</Menu>

<Menu name="document">
  <text>&amp;Composition</text>
  <Action name="add_tempo"/>
  <Action name="add_time_signature"/>
</Menu>

<Menu name="segment">
  <text>Se&amp;gment</text>
  <Menu name="open-with">
	   <text>Edit &amp;With</text>
	   <Action name="open_in_event_list"/>
	   <Action name="open_in_matrix"/>
	   <Action name="open_in_percussion_matrix"/>
  </Menu>
  <Separator/>
  <Action name="add_clef"/>
  <Action name="add_key_signature"/>
  <Separator/>
  <Action name="add_sustain_down"/>
  <Action name="add_sustain_up"/>
  <Separator/>
  <Action name="set_segment_start"/>
  <Action name="set_segment_duration"/>
  <Separator/>
  <Action name="transpose_segment"/>
  <Action name="switch_preset"/>
</Menu>


<Menu name="Notes">
	<text>N&amp;ote</text>
	<Separator/>
	<Menu name="Marks">
		<text>Mar&amp;ks</text>
		<Action name="add_accent"/>
		<Action name="add_tenuto"/>
		<Action name="add_staccato"/>
		<Action name="add_staccatissimo"/>
		<Action name="add_marcato"/>
		<Action name="add_text_sf"/>
		<Action name="add_text_rf"/>
		<Separator/>
		<Action name="add_trill"/>
		<Action name="add_long-trill"/>
		<Action name="add_trill-line"/>
		<Action name="add_turn"/>
		<Action name="add_mordent"/>
		<Action name="add_mordent-inverted"/>
		<Action name="add_mordent-long"/>
		<Action name="add_mordent-long-inverted"/>
		<Separator/>
		<Action name="add_up-bow"/>
		<Action name="add_down-bow"/>
		<Action name="add_pause"/>
		<Separator/>
		<Action name="add_text_mark"/>
		<Separator/>
		<Action name="remove_marks"/>
	</Menu>
	<Menu name="ornaments">
		<text>&amp;Ornaments</text>
		<Action name="use_ornament"/>
		<Action name="remove_ornament"/>
		<Action name="make_ornament"/>
		<Action name="ornament_actionmenu"/>
	</Menu>
		<Separator/>
	<Menu name="Fingering">
		<text>&amp;Fingerings</text>
		<Action name="remove_fingering_marks"/>
		<Separator/>
		<Action name="add_fingering_0"/>
		<Action name="add_fingering_1"/>
		<Action name="add_fingering_2"/>
		<Action name="add_fingering_3"/>
		<Action name="add_fingering_4"/>
		<Action name="add_fingering_5"/>
		<Action name="add_fingering_plus"/>
		<Action name="add_fingering_mark"/>
	</Menu>
	<Menu name="Slashes">
		<text>S&amp;lashes</text>
		<Action name="slashes_0"/>
		<Action name="slashes_1"/>
		<Action name="slashes_2"/>
		<Action name="slashes_3"/>
		<Action name="slashes_4"/>
		<Action name="slashes_5"/>
	</Menu>
	<Action name="note_style_actionmenu"/>
		<Separator/>
	<Menu name="Respell">
		<text>&amp;Accidentals</text>
		<Action name="respell_restore"/>
		<Separator/>
		<Action name="respell_doubleflat"/>
		<Action name="respell_flat"/>
		<Action name="respell_natural"/>
		<Action name="respell_sharp"/>
		<Action name="respell_doublesharp"/>
		<Separator/>
		<Action name="show_cautionary"/>
		<Action name="cancel_cautionary"/>
	</Menu>
	<Separator/>
		<Action name="stems_up"/>
		<Action name="stems_down"/>
		<Action name="restore_stems"/>
</Menu>

<Menu name="Phrase">
	<text>&amp;Phrase</text>
	<Action name="make_chord"/>
	<Separator/>
		    <Action name="beam"/>
		    <Action name="auto_beam"/>
		    <Action name="break_group"/>
	    <Action name="remove_indications"/>
	<Separator/>
		<Action name="tuplet"/>
		<Action name="simple_tuplet"/>
		<Action name="break_tuplets"/>
	<Separator/>
	<Action name="grace"/>
	    <Separator/>
  		<Action name="slur"/>
		<Action name="phrasing_slur"/>
	  <!--  <Action name="glissando"/> -->
	  	<Menu name="Slurs">
			<text>Slur &amp;Position</text>
			<Action name="restore_slurs"/>
			<Separator/>
	  		<Action name="slurs_above"/>
			<Action name="slurs_below"/>
		</Menu>
		<Separator/>
		<Action name="tie_notes"/>
		<Action name="untie_notes"/>
	  	<Menu name="Ties">
			<text>Tie &amp;Position</text>
			<Action name="restore_ties"/>
			<Separator/>
	  		<Action name="ties_above"/>
			<Action name="ties_below"/>
		</Menu>
		<Separator/>
		<Action name="crescendo"/>
		<Action name="decrescendo"/>
		<Separator/>
	    <Menu name="octaves">
		    <text>&amp;Octaves</text>
		    <Action name="octave_2up"/>
		    <Action name="octave_up"/>
		    <Action name="octave_down"/>
		    <Action name="octave_2down"/>
	    </Menu>
</Menu>

<Menu name="Adjust">
	<text>Ad&amp;just</text>
	<Menu name="rests">
		<text>R&amp;ests</text>
		<Action name="normalize_rests"/>
		<Action name="collapse_rests_aggressively"/>
	</Menu>
	<Menu name="transform_notes">
		<text>&amp;Notes</text>
	        <Action name="collapse_notes"/>
	        <Action name="make_notes_viable"/>
	        <Action name="de_counterpoint"/>
	</Menu>
	<Separator/>
	<Menu name="quantize_notes">
		<text>&amp;Quantize</text>
	        <Action name="quantize"/>
  	        <Action name="fix_quantization"/>
	        <Action name="remove_quantization"/>
	</Menu>
 	<Action name="interpret"/>
        <Separator/>
	<Menu name="Rescale">
	      <text>Rescale</text>
	      <Action name="halve_durations"/>
	      <Action name="double_durations"/>
	      <Separator/>
	      <Action name="rescale"/>
	</Menu>
        <Menu name="Transpose">
	      <text>Trans&amp;pose</text>
	      <Action name="transpose_up"/>
	      <Action name="transpose_down"/>
	      <Separator/>
	      <Action name="transpose_up_octave"/>
	      <Action name="transpose_down_octave"/>
	      <Separator/>
	      <Action name="general_transpose"/>
	      <Action name="general_diatonic_transpose"/>
	</Menu>
        <Menu name="Convert">
	      <text>Convert</text>
	      <Action name="invert"/>
 	      <Action name="retrograde"/>
	      <Action name="retrograde_invert"/>
	</Menu>
		<Separator/>
	<Menu name="fine_positioning">
		<text>&amp;Fine Positioning</text>
		<Action name="fine_position_restore"/>
 		<Separator/>
		<Action name="fine_position_left"/>
		<Action name="fine_position_right"/>
		<Action name="fine_position_up"/>
		<Action name="fine_position_down"/>
	</Menu>
	<Menu name="fine_timing">
		<text>Fine Ti&amp;ming</text>
	        <Action name="jog_left"/>
	        <Action name="jog_right"/>
	</Menu>
	<Menu name="visibility">
		<text>&amp;Visibility</text>
		<Action name="make_invisible"/>
		<Action name="make_visible"/>
	</Menu>
</Menu>

<Menu name="Tools">
  <text>&amp;Tools</text>
	<Action name="select"/>
	<Action name="erase"/>
      <Menu noMerge="1" name="Notes">
	    <text>&amp;Notes</text>
	    <!-- Reference names, but with _ instead of - -->
	    <Action name="breve" />
	    <Action name="semibreve" />
	    <Action name="minim" />
	    <Action name="crotchet" />
	    <Action name="quaver" />
	    <Action name="semiquaver" />
	    <Action name="demisemi" />
	    <Action name="hemidemisemi" />
	    <Separator/>
	    <Action name="dotted_semibreve" />
	    <Action name="dotted_minim" />
	    <Action name="dotted_crotchet" />
	    <Action name="dotted_quaver" />
	    <Action name="dotted_semiquaver" />
	    <Action name="dotted_demisemi" />
	    <Separator/>
	    <Action name="switch_from_rest_to_note" />
      </Menu>

      <Menu noMerge="1" name="Rests">
	    <text>&amp;Rests</text>
	    <!-- Reference names, but with _ instead of - -->
	    <Action name="rest_breve" />
	    <Action name="rest_semibreve" />
	    <Action name="rest_minim" />
	    <Action name="rest_crotchet" />
	    <Action name="rest_quaver" />
	    <Action name="rest_semiquaver" />
	    <Action name="rest_demisemi" />
	    <Action name="rest_hemidemisemi" />
	    <Separator/>
	    <Action name="dotted_rest_semibreve" />
	    <Action name="dotted_rest_minim" />
	    <Action name="dotted_rest_crotchet" />
	    <Action name="dotted_rest_quaver" />
	    <Action name="dotted_rest_semiquaver" />
	    <Action name="dotted_rest_demisemi" />
	    <Separator/>
	    <Action name="switch_from_note_to_rest" />
      </Menu>

      <Menu noMerge="1" name="Accidentals">
	    <text>&amp;Accidentals</text>
	    <Action name="no_accidental" />
	    <Action name="follow_accidental" />
	    <Action name="sharp_accidental" />
	    <Action name="flat_accidental" />
	    <Action name="natural_accidental" />
	    <Action name="double_sharp_accidental" />
	    <Action name="double_flat_accidental" />
      </Menu>

      <Menu noMerge="1" name="Clefs">
	    <text>&amp;Clefs</text>
	    <Action name="treble_clef" />
	    <Action name="alto_clef" />
	    <Action name="tenor_clef" />
	    <Action name="bass_clef" />
      </Menu>
  <Separator/>
  <Action name="text" />
  <Action name="guitarchord" />
  <Separator/>
  <Menu name="Move">
  	<text>Local &amp;Cursor</text>
	<Action name="cursor_back"/>
	<Action name="cursor_forward"/>
	<Action name="cursor_back_bar"/>
	<Action name="cursor_forward_bar"/>
	<Action name="cursor_start"/>
	<Action name="cursor_end"/>
	<Separator/>
	<Action name="extend_selection_backward"/>
	<Action name="extend_selection_forward"/>
	<Action name="preview_selection"/>
	<Action name="clear_loop"/>
	<!-- not implemented yet
	<Separator/>
	<Action name="move_selection_left"/>
	<Action name="move_selection_right"/>
	<Action name="copy_selection_left"/>
	<Action name="copy_selection_right"/>
	-->
	<Separator/>
	<Action name="cursor_to_playback_pointer"/>
	<Action name="playback_pointer_to_cursor"/>
	<Separator/>
	<Action name="cursor_up_staff"/>
	<Action name="cursor_down_staff"/>
  	<Action name="cursor_prior_segment"/>
  	<Action name="cursor_next_segment"/>
  </Menu>

  <Menu name="Transport">
        <text>T&amp;ransport</text>
        <Action name="play"/> 
        <Action name="stop"/> 
  	<Action name="playback_pointer_back_bar"/>
	<Action name="playback_pointer_forward_bar"/>
	<Action name="playback_pointer_start"/>
	<Action name="playback_pointer_end"/>
	<Action name="playback_pointer_to_cursor" />
	<Action name="cursor_to_playback_pointer" />
	<Action name="toggle_solo"/>
	<Action name="toggle_tracking"/>
	<Action name="panic"/>
  </Menu>

  <Separator/>

  <Action name="insert_note_actionmenu"/>
  <Action name="insert_rest"/>

  <!--
  <Action name="insert_melody_mode"/>
  <Action name="insert_chord_mode"/>
  -->

  <Action name="chord_mode"/>
  <Action name="triplet_mode"/>
  <Action name="grace_mode"/>

  <Separator/>
  <Action name="toggle_step_by_step"/>

</Menu>

<Menu  name="settings">
	<text>&amp;Settings</text>
	<Menu name="toolbars"                            append="show_merge"  >
		<text>&amp;Toolbars</text>
		<Action name="options_show_toolbar"      append="show_merge" />
		<Action name="show_tools_toolbar"        append="show_merge" />
		<Action name="show_notes_toolbar"        append="show_merge" />
		<Action name="show_rests_toolbar"        append="show_merge" />
		<Action name="show_accidentals_toolbar"  append="show_merge" />
		<Action name="show_clefs_toolbar"        append="show_merge" />
		<Action name="show_marks_toolbar"        append="show_merge" />
		<Action name="show_group_toolbar"        append="show_merge" />
		<Action name="show_transport_toolbar"    append="show_merge" />
		<Action name="show_layout_toolbar"       append="show_merge" />
		<Action name="show_meta_toolbar"         append="show_merge" />
		<Action name="options_show_statusbar"    append="show_merge" /> 
	</Menu>
		<Menu name="rulers"                      append="show_merge"  >
		<text>&amp;Rulers</text>
		<Action name="show_chords_ruler"         append="show_merge" />
		<Action name="show_raw_note_ruler"       append="show_merge" />
		<Action name="show_tempo_ruler"          append="show_merge" />
	</Menu>
	<Separator                                   append="show_merge" />
	<Action name="show_annotations"              append="show_merge" />
	<Action name="show_lilypond_directives"      append="show_merge" />
</Menu> 

</MenuBar>

<ToolBar name="Tools Toolbar">
  <text>Tools Toolbar</text>
  <Action name="select" />
  <Action name="erase" />
  <Action name="text" />
  <Action name="guitarchord" />
  <Separator/>
  <Action name="toggle_step_by_step"/>
  <Action name="filter_selection"/>
  <Action name="quantize"/>
</ToolBar>

<ToolBar name="Transport Toolbar">
  <text>Transport Toolbar</text>
  <Action name="playback_pointer_start" />
  <Action name="playback_pointer_back_bar" />
  <Action name="play" />
  <Action name="playback_pointer_forward_bar" />
  <Action name="playback_pointer_end" />
  <Action name="stop" />
  <Action name="playback_pointer_to_cursor" />
  <Action name="cursor_to_playback_pointer" />
  <Action name="toggle_solo"/>
  <Action name="toggle_tracking"/>
  <Action name="panic"/>
</ToolBar>

<ToolBar name="Notes Toolbar" newline="true" hidden="false" position="left">
  <text>Notes Toolbar</text>

  <!-- Reference names, but with _ instead of - -->
  <Action name="breve" />
  <Action name="semibreve" />
  <Action name="minim" />
  <Action name="crotchet" />
  <Action name="quaver" />
  <Action name="semiquaver" />
  <Action name="demisemi" />
  <Action name="hemidemisemi" />
  <Separator/>
  <Action name="dotted_semibreve" />
  <Action name="dotted_minim" />
  <Action name="dotted_crotchet" />
  <Action name="dotted_quaver" />
  <Action name="dotted_semiquaver" />
  <Action name="dotted_demisemi" />

  <Separator/>

  <Action name="chord_mode"/>
  <Action name="triplet_mode" />
  <Action name="grace_mode" />
</ToolBar>

<ToolBar name="Rests Toolbar" hidden="false" position="left">
  <text>Rests Toolbar</text>

  <!-- Reference names, but with _ instead of - -->
  <Action name="rest_breve" />
  <Action name="rest_semibreve" />
  <Action name="rest_minim" />
  <Action name="rest_crotchet" />
  <Action name="rest_quaver" />
  <Action name="rest_semiquaver" />
  <Action name="rest_demisemi" />
  <Action name="rest_hemidemisemi" />
  <Separator/>
  <Action name="dotted_rest_semibreve" />
  <Action name="dotted_rest_minim" />
  <Action name="dotted_rest_crotchet" />
  <Action name="dotted_rest_quaver" />
  <Action name="dotted_rest_semiquaver" />
  <Action name="dotted_rest_demisemi" />

</ToolBar>

<ToolBar name="Clefs Toolbar" hidden="false" position="left" newline="true">
  <text>Clefs Toolbar</text>

  <Action name="treble_clef" />
  <Action name="alto_clef" />
  <Action name="tenor_clef" />
  <Action name="bass_clef" />
</ToolBar>

<ToolBar name="Accidentals Toolbar" hidden="false" position="left">
  <text>Accidentals Toolbar</text>

  <Action name="no_accidental" />
  <Action name="follow_accidental" />
  <Action name="sharp_accidental" />
  <Action name="flat_accidental" />
  <Action name="natural_accidental" />
  <Action name="double_sharp_accidental" />
  <Action name="double_flat_accidental" />
</ToolBar>

<ToolBar name="Group Toolbar" hidden="false" position="right">
  <text>Group Toolbar</text>

  <Action name="make_chord"/>
  <Action name="beam"/>
  <Action name="break_group"/>
  <Separator/>
  <Action name="tuplet"/>
  <Action name="simple_tuplet"/>
<!--  <Action name="grace"/> -->
  <Separator/>
  <Action name="slur"/>
<!--  <Action name="glissando"/> -->
  <Action name="tie_notes"/>
  <Action name="de_counterpoint"/>
  <Separator/>
  <Action name="crescendo"/>
  <Action name="decrescendo"/>
  <Action name="octave_up"/>
</ToolBar>

<ToolBar name="Marks Toolbar" newline="true" hidden="false" position="right">
  <text>Marks Toolbar</text>

    <Action name="add_accent"/>
    <Action name="add_tenuto"/>
    <Action name="add_staccato"/>
    <Action name="add_staccatissimo"/>
    <Action name="add_marcato"/>
    <Action name="add_text_sf"/>
    <Action name="add_text_rf"/>
    <Action name="add_trill"/>
    <Action name="add_long-trill"/>
    <Action name="add_turn"/>
    <Action name="add_pause"/>
    <Action name="add_up-bow"/>
    <Action name="add_down-bow"/>
    <Action name="add_mordent"/>
    <Action name="add_mordent-inverted"/>
    <Action name="add_mordent-long"/>
    <Action name="add_mordent-long-inverted"/>
    <Action name="add_text_mark"/>
</ToolBar>

<ToolBar name="Meta Toolbar" newline="true" hidden="true">
  <text>Meta Toolbar</text>

  <Action name="show_notes_toolbar"        />
  <Action name="show_rests_toolbar"        />
  <Action name="show_accidentals_toolbar"  />
  <Action name="show_clefs_toolbar"        />
  <Action name="show_marks_toolbar"        />
  <Action name="show_group_toolbar"        />
</ToolBar>

<ToolBar name="Layout Toolbar" hidden="true">
  <text>Layout Toolbar</text>
  <Action name="linear_mode"               />
  <Action name="continuous_page_mode"      />
  <Action name="multi_page_mode"           />
<!-- the remainder filled at runtime -->
</ToolBar>

<Menu noMerge="1" name="controller_events_ruler_menu">
   <Text>Control Ruler actions</Text>
   <Action name="insert_control_ruler_item"/>
   <Action name="erase_control_ruler_item"/>
   <Action name="clear_control_ruler_item"/>
   <Separator/>
   <Action name="start_control_line_item"/>
   <Separator/>
   <Action name="flip_control_events_forward"/>
   <Action name="flip_control_events_back"/>
</Menu>

<Menu noMerge="1" name="property_ruler_menu">
   <Text>Property Ruler actions</Text>
   <Action name="draw_property_line"/>
   <Action name="select_all_properties"/>
   <Separator/>
   <Action name="flip_control_events_forward"/>
   <Action name="flip_control_events_back"/>
</Menu>



<State name="note_insert_tool_current">
  <enable>
    <Action name="insert_rest"/>
    <Action name="insert_note_actionmenu"/>
    <Action name="switch_from_note_to_rest"/>
  </enable>
</State>

<State name="rest_insert_tool_current">
  <enable>
    <Action name="insert_rest"/>
    <Action name="switch_from_rest_to_note"/>
  </enable>
</State>

<State name="have_selection">
  <enable>
    <Action name="edit_cut"/>
    <Action name="edit_copy"/>
    <Action name="delete"/>
    <Action name="cut_and_close"/>
    <Action name="debug_dump"/>
    <Action name="preview_selection"/>
    <Action name="normalize_rests"/>
    <Action name="clear_selection"/>
    <Action name="filter_selection"/>
    <Action name="jog_left"/>
    <Action name="jog_right"/>
    <Action name="slurs_above"/>
    <Action name="slurs_below"/>
    <Action name="restore_slurs"/>
    <Action name="ties_above"/>
    <Action name="ties_below"/>
    <Action name="restore_ties"/>
    <Action name="fine_position_left"/>
    <Action name="fine_position_right"/>
    <Action name="fine_position_up"/>
    <Action name="fine_position_down"/>
    <Action name="fine_position_restore"/>
	<Action name="tuplet"/>
	<Action name="simple_tuplet"/>
	<Action name="break_tuplets"/>
	<Action name="make_invisible"/>
	<Action name="make_visible"/>
    <Action name="move_events_up_staff"/>
    <Action name="move_events_down_staff"/>
  </enable>
</State>

<State name="have_rests_in_selection">
  <enable>
    <Action name="note_style_actionmenu"/>
    <Action name="remove_marks"/>
    <Action name="add_pause"/>
    <Action name="add_text_mark"/>
<!--    <Action name="normalize_rests"/> -->
    <Action name="collapse_rests_aggressively"/>
    <Menu name="Adjust"/>
  </enable>
</State>

<State name="have_notes_in_selection">
  <enable>
    <Action name="respell_doubleflat"/>
    <Action name="respell_flat"/>
    <Action name="respell_natural"/>
    <Action name="respell_sharp"/>
    <Action name="respell_doublesharp"/>
    <Action name="respell_restore"/>
    <Action name="show_cautionary"/>
    <Action name="cancel_cautionary"/>
    <Action name="make_chord"/>
    <Action name="beam"/>
    <Action name="auto_beam"/>
    <Action name="break_group"/>
<!--    <Action name="break_tuplets"/> -->
<!--    <Action name="grace"/> -->
<!--    <Action name="ungrace"/> -->
    <Action name="slur"/>
  <Action name="phrasing_slur"/>
    <Action name="crescendo"/>
    <Action name="decrescendo"/>
<!--  <Action name="glissando"/> -->
     <Action name="octave_2up"/>
     <Action name="octave_up"/>
     <Action name="octave_down"/>
     <Action name="octave_2down"/>
<!--    <Action name="remove_indications"/> -->
    <Action name="stems_up"/>
    <Action name="stems_down"/>
    <Action name="restore_stems"/>
    <Action name="slashes_0"/>
    <Action name="slashes_1"/>
    <Action name="slashes_2"/>
    <Action name="slashes_3"/>
    <Action name="slashes_4"/>
    <Action name="slashes_5"/>
    <Action name="note_style_actionmenu"/>
    <Action name="remove_marks"/>
    <Action name="remove_fingering_marks"/>
    <Action name="add_fingering_0"/>
    <Action name="add_fingering_1"/>
    <Action name="add_fingering_2"/>
    <Action name="add_fingering_3"/>
    <Action name="add_fingering_4"/>
    <Action name="add_fingering_5"/>
    <Action name="add_fingering_plus"/>
    <Action name="add_fingering_mark"/>
    <Action name="add_accent"/>
    <Action name="add_tenuto"/>
    <Action name="add_staccato"/>
    <Action name="add_staccatissimo"/>
    <Action name="add_marcato"/>
    <Action name="add_text_sf"/>
    <Action name="add_text_rf"/>
    <Action name="add_trill"/>
    <Action name="add_long-trill"/>
    <Action name="add_trill-line"/>
    <Action name="add_turn"/>
    <Action name="add_pause"/>
    <Action name="add_up-bow"/>
    <Action name="add_down-bow"/>
    <Action name="add_mordent"/>
    <Action name="add_mordent-inverted"/>
    <Action name="add_mordent-long"/>
    <Action name="add_mordent-long-inverted"/>
    <Action name="add_text_mark"/>
    <Action name="make_ornament"/>
    <Action name="remove_ornament"/>
    <Action name="use_ornament"/>
    <Action name="collapse_notes"/>
    <Action name="tie_notes"/>
    <Action name="untie_notes"/>
    <Action name="make_notes_viable"/>
    <Action name="de_counterpoint"/>
    <Action name="quantize"/>
    <Action name="interpret"/>
    <Action name="fix_quantization"/>
    <Action name="remove_quantization"/>
    <Action name="transpose_up"/>
    <Action name="transpose_down"/>
    <Action name="transpose_up_octave"/>
    <Action name="transpose_down_octave"/>
    <Action name="general_transpose"/>
    <Action name="general_diatonic_transpose"/>
    <Action name="invert"/>
    <Action name="retrograde"/>
    <Action name="retrograde_invert"/>
    <Action name="halve_durations"/>
    <Action name="double_durations"/>
    <Action name="rescale"/>
    <Menu name="Stems"/>
    <Menu name="Slashes"/>
    <Menu name="Transpose"/>
    <Menu name="Convert"/>
<Menu name="Notes"/>
<Menu name="beams"/>
<Menu name="octaves"/>
<Menu name="indications"/>
<Menu name="Slurs"/>
<Menu name="Ties"/>
<Menu name="Marks"/>
<Menu name="ornaments"/>
<Menu name="Fingering"/>
<Menu name="Slashes"/>
<Menu name="Adjust"/>
<Menu name="Respell"/>
<Menu name="triplets"/>
<Menu name="Stems"/>
<Menu name="rests"/>
<Menu name="transform_notes"/>
<Menu name="quantize_notes"/>
  </enable>
</State>

<State name="have_clipboard_single_segment">
  <enable>
    <Action name="edit_paste"/>
    <Action name="general_paste"/>
  </enable>
</State>

<State name="have_multiple_staffs">
  <enable>
    <Action name="cursor_up_staff"/>
    <Action name="cursor_down_staff"/>
    <Action name="cursor_prior_segment"/>
    <Action name="cursor_next_segment"/>
    <Action name="move_events_up_staff"/>
    <Action name="move_events_down_staff"/>
  </enable>
</State>

<State name="linear_mode">
  <enable>
    <Action name="show_velocity_control_ruler"/>
    <Action name="show_controller_events_ruler"/>
    <Menu name="add_control_ruler"/>
    <Action name="add_control_ruler"/>
    <Action name="show_chords_ruler"/>
    <Action name="show_raw_note_ruler"/>
    <Action name="show_tempo_ruler"/>
  </enable>
</State>

<State name="have_note_events_in_segment">
  <enable>
   <Action name="draw_property_line"/>
  </enable>
</State>

<State name="have_control_ruler">
  <enable>
    <Action name="show_control_rulers"/>
  </enable>
</State>

<State name="have_controller_item_selected">
  <enable>
    <Action name="erase_control_ruler_item"/>
  </enable>
  <disable>
   <Action name="insert_control_ruler_item"/>
  </disable>
</State>

</kpartgui>
