/*
    Rosegarden
    A MIDI and audio sequencer and musical notation editor.

    This program is Copyright 2000-2008
        Guillaume Laurent   <glaurent@telegraph-road.org>,
        Chris Cannam        <cannam@all-day-breakfast.com>,
        Richard Bown        <richard.bown@ferventsoftware.com>

    The moral rights of Guillaume Laurent, Chris Cannam, and Richard
    Bown to claim authorship of this work have been asserted.

    Other copyrights also apply to some parts of this work.  Please
    see the AUTHORS file and individual file headers for details.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/

#ifndef _RG_INTERVALDIALOG_H_
#define _RG_INTERVALDIALOG_H_

#include <kdialogbase.h>
#include <vector>
#include "gui/application/RosegardenDCOP.h"
#include "gui/widgets/DiatonicPitchChooser.h"


class TQWidget;
class KComboBox;
class TQRadioButton;
class TQCheckBox;


namespace Rosegarden
{

class RosegardenGUIDoc;


class IntervalDialog : public KDialogBase
{
    TQ_OBJECT
  
public:
    IntervalDialog(TQWidget *parent, bool askChangeKey = false, bool askTransposeSegmentBack = false);
    
    // Distance in semitones
    int getChromaticDistance();

    // Distance in steps
    int getDiatonicDistance();

    // Transpose within key or change the key?
    bool getChangeKey();
    
    // Transpose the segment itself in the opposite direction?
    bool getTransposeSegmentBack();
    
    static TQString getIntervalName(int intervalDiatonic, int intervalChromatic);
    
public slots:
    void slotSetReferenceNote(int,int,int);
    void slotSetTargetNote(int,int,int);
    
private:
    int getOctaveDistance();
    int getStepDistanceChromatic();
    int getAccidentalCorrectionChromatic();
    
    DiatonicPitchChooser *m_referencenote;
    DiatonicPitchChooser *m_targetnote;

    TQRadioButton *m_transposeWithinKey;
    TQRadioButton *m_transposeChangingKey;
    bool changeKey;

	TQCheckBox *m_transposeSegmentBack;

    int intervalChromatic;
    int intervalDiatonic;
    TQLabel *m_intervalLabel;

};


}

#endif
