<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="Rosegarden::EventList" version="1230">

<MenuBar>

<Menu name="file" append="new_merge">
</Menu>

<Menu name="edit" append="new_merge">
  <Action name="insert"/>
  <Action name="delete"/>
  <Action name="edit_simple"/>
  <Action name="edit_advanced"/>
  <Separator/>
  <Action name="select_all"/>
  <Action name="clear_selection"/>
  <Action name="filter_selection"/>
  <Separator/>
  <Action name="set_segment_start"/>
  <Action name="set_segment_duration"/>
</Menu>

<Menu name="view"><text>&amp;View</text>
  <Action name="time_musical"/>
  <Action name="time_real"/>
  <Action name="time_raw"/>
</Menu>

<Menu name="segment">
  <text>Se&amp;gment</text>
  <Menu name="open-with">
           <text>Edit &amp;With</text>
           <Action name="open_in_matrix"/>
           <Action name="open_in_notation"/>
  </Menu>
</Menu>
</MenuBar>

<ToolBar name="Actions Toolbar">
  <text>Actions Toolbar</text>
  <Action name="insert"/>
  <Action name="delete"/>
  <Action name="edit_simple"/>
  <Action name="edit_advanced"/>
  <Action name="filter_selection"/>
</ToolBar>

<ToolBar name="Time Toolbar">
  <text>Time Toolbar</text>
  <Action name="time_musical"/>
  <Action name="time_real"/>
  <Action name="time_raw"/>
</ToolBar>

</kpartgui>

<ToolBar name="Transport Toolbar">
  <text>Transport Toolbar</text>

  <Action name="playback_pointer_start" />
  <Action name="playback_pointer_back_bar" />
  <Action name="play" />
  <Action name="playback_pointer_forward_bar" />
  <Action name="playback_pointer_end" />
  <Action name="stop" />
  <Action name="playback_pointer_to_cursor" />
  <Action name="cursor_to_playback_pointer" />
  <Action name="toggle_solo"/>
</ToolBar>

<ToolBar name="Actions Toolbar">
  <text>Actions Toolbar</text>
</ToolBar>

<ToolBar name="Zoom Toolbar">
  <text>Zoom Toolbar</text>
</ToolBar>

<State name="have_selection">
  <enable>
    <Action name="edit_cut"/>
    <Action name="edit_copy"/>
    <Action name="delete"/>
    <Action name="edit_simple"/>
    <Action name="edit_advanced"/>
    <Action name="clear_selection"/>
    <Action name="filter_selection"/>
  </enable>
</State>

<State name="have_clipboard_single_segment">
  <enable>
    <Action name="edit_paste"/>
  </enable>
</State>

<State name="have_control_ruler">
  <enable>
    <Action name="show_control_rulers"/>
  </enable>
</State>

</kpartgui>
 
