<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="Rosegarden::MatrixTool" version="1230">

<Menu name="MatrixPainter">
  <Action name="select" />
  <Action name="move"/>
  <Action name="erase"/>
  <Action name="resize"/>
<!-- <Separator/> -->
<!-- Defunct: <Action name="halve_durations"/> -->
<!-- Defunct: <Action name="double_durations"/> -->
</Menu>

<!-- Defunct: <State name="have_selection"> -->
<!--  <enable> -->
<!--    <Action name="halve_durations"/> -->
<!--    <Action name="double_durations"/> -->
<!--  </enable> -->
<!--</State> -->

</kpartgui>
