/*
    Rosegarden
    A MIDI and audio sequencer and musical notation editor.

    This program is Copyright 2000-2008
        Guillaume Laurent   <glaurent@telegraph-road.org>,
        Chris Cannam        <cannam@all-day-breakfast.com>,
        Richard Bown        <richard.bown@ferventsoftware.com>

    The moral rights of Guillaume Laurent, Chris Cannam, and Richard
    Bown to claim authorship of this work have been asserted.

    Other copyrights also apply to some parts of this work.  Please
    see the AUTHORS file and individual file headers for details.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/

#ifndef _RG_PITCHRULER_H_
#define _RG_PITCHRULER_H_

#include <tqsize.h>
#include <tqwidget.h>




namespace Rosegarden
{



class PitchRuler : public TQWidget
{
    TQ_OBJECT
  
public:
    PitchRuler(TQWidget *parent);

    virtual TQSize sizeHint() const;
    virtual TQSize minimumSizeHint() const;

    virtual void drawHoverNote(int evPitch) = 0;

signals:

    /**
     * A pitch has been clicked.
     * y is the simple event y-coordinate.
     * If the user is in the middle of dragging, repeating will be set.
     */
    void keyPressed(unsigned int y, bool repeating);

    /**
     * A pitch has been clicked with the selection modifier pressed.
     * y is the simple event y-coordinate.
     * If the user is in the middle of dragging, repeating will be set.
     */
    void keySelected(unsigned int y, bool repeating);

    /**
     * Emitted when the mouse cursor moves to a different pitch when
     * not clicking or selecting.
     * y is the simple event y-coordinate.
     */
    void hoveredOverKeyChanged(unsigned int y);
};


}

#endif
