<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="Rosegarden" version="1230">

<MenuBar>
  <Menu name="file">
    <!-- these two would like to be open_merge, but that sticks them at
	 the very bottom in older KDE3 versions: -->
    <Menu name="import" append="new_merge"><text>&amp;Import</text>
      <Action name="file_import_project" />
      <Separator/>
      <Action name="file_import_midi" />
      <Action name="file_import_rg21" />
      <Action name="file_import_hydrogen" />
    </Menu>
    <Menu name="merge" append="new_merge"><text>&amp;Merge</text>
      <Action name="file_merge" />
      <Action name="file_merge_midi" />
      <Action name="file_merge_rg21" />
      <Action name="file_merge_hydrogen" />
    </Menu>
    <!-- this one would like to be save_merge, but that sticks it at
	 the very bottom in older KDE3 versions: -->
    <Menu name="export" append="new_merge"><text>&amp;Export</text>
      <Action name="file_export_project" />
      <Separator/>
      <Action name="file_export_lilypond" />
      <Separator/>
      <Action name="file_export_midi" />
      <Separator/>
      <Action name="file_export_csound" />
      <Action name="file_export_mup" />
      <Action name="file_export_musicxml" />
    </Menu>
    <Action name="file_print_lilypond" append="print_merge" />
    <Action name="file_preview_lilypond" append="print_merge" />
  <Separator/>
        <Action name="audio_manager"/>
  <Separator/>
  <Action name="file_show_playlist" />
  </Menu>
  <Menu name="edit">
        <Action name="delete"/>
        <Separator/>
        <Action name="cut_range"/>
        <Action name="copy_range"/>
        <Action name="paste_range"/>
        <Action name="insert_range"/>
        <Separator/>
        <Action name="select_all"/>
  </Menu>
  <Menu name="document">
  	<text>&amp;Composition</text>
	    <Action name="add_tempo"/>
	    <Action name="add_time_signature"/>
	    <Action name="edit_tempos"/>
	    <Separator/>
	    <Action name="set_tempo_to_segment_length"/>
	    <Action name="groove_quantize"/>
        <Separator/>
        <Action name="change_composition_length"/>
        <Separator/>
        <Action name="edit_markers"/>
        <Action name="edit_doc_properties"/>
        <Separator/>
        <Action name="set_quick_marker"/>
        <Action name="jump_to_quick_marker"/>
  </Menu>
  <Menu name="Studio">
	<text>&amp;Studio</text>
	<Action name="audio_mixer"/>
	<Action name="midi_mixer"/>
	<Separator/>
	<Action name="manage_devices"/>
	<Action name="manage_synths"/>
	<Action name="manage_metronome"/>
	<Separator/>
	<Action name="modify_midi_filters"/>
	<Action name="enable_midi_routing"/>
	<Separator/>
	      <Action name="reset_midi_network"/>
	<Separator/>
	<Action name="load_studio"/>
	<Action name="load_default_studio"/>
	<Action name="save_default_studio"/>
  </Menu>
  <Menu name="Segments">
        <text>Se&amp;gments</text>
	<Menu name="edit_with">
  	    <text>Edit &amp;With</text>
	    <Action name="edit_default"/>
	    <Separator/>
	    <Action name="edit_matrix"/>
	    <Action name="edit_percussion_matrix"/>
	    <Action name="edit_notation"/>
	    <Action name="edit_event_list"/>
	</Menu>
        <Separator/>
        <Action name="relabel_segment"/>
        <Action name="quantize_selection"/>
		<Action name="repeat_quantize"/>
		<Action name="transpose"/>
        <Separator/>
        <Action name="rescale"/>
        <Action name="set_segment_start"/>
        <Action name="set_segment_duration"/>
	<Separator/>
        <Action name="jog_left"/>
        <Action name="jog_right"/>
	<Separator/>
        <Action name="join_segments"/>
	<Menu name="split">
	<text>&amp;Split</text>
	    <Action name="auto_split"/>
	    <Action name="split_by_pitch"/>
	    <Action name="split_by_recording"/>
	    <Action name="split_at_time"/>
	</Menu>  
        <Action name="repeats_to_real_copies"/>
        <Separator/>
        <Action name="manage_trigger_segments"/>
  </Menu>
  <Menu name="Tracks">
        <text>Trac&amp;ks</text>
        <Action name="add_track"/>
        <Action name="add_tracks"/>
        <Action name="delete_track"/>
	<Separator/>
        <Action name="move_track_down"/>
        <Action name="move_track_up"/>
	<Separator/>
        <Action name="select_next_track"/>
        <Action name="select_previous_track"/>
        <Separator/>
        <Action name="toggle_mute_track"/>
        <Action name="toggle_arm_track"/>
        <Separator/>
        <Action name="unmute_all_tracks"/>
        <Action name="mute_all_tracks"/>
        <Separator/>
        <Menu name="set_track_instrument"><text>Set &amp;Instrument</text></Menu>
        <Action name="remap_instruments"/>
  </Menu>
  <Menu name="Tools">
  <text>&amp;Tools</text>
	    <Action name="select"/>
	    <Action name="draw"/>
	    <Action name="erase"/>
	    <Action name="move"/>
	    <Action name="resize"/>
	    <Action name="split"/>
      <Separator/>
      <Menu name="Transport">
	    <text>T&amp;ransport</text>
	    <Action name="play"/> 
	    <Action name="stop"/> 
	    <Action name="fast_forward"/> 
	    <Action name="rewind"/> 
	    <Action name="record"/>
	    <Action name="recordtoggle"/>
	    <Action name="toggle_tracking"/>
	    <Action name="panic"/>
      </Menu>
  </Menu>
  <Menu  name="settings">
	<Menu name="toolbars" append="show_merge"><text>&amp;Toolbars</text>
	     <Action name="show_stock_toolbar"         append="show_merge" />
	     <Action name="show_tools_toolbar"         append="show_merge" />
	     <Action name="show_tracks_toolbar"        append="show_merge" />
	     <Action name="show_editors_toolbar"       append="show_merge" />
	     <Action name="show_transport_toolbar"     append="show_merge" />
	     <Action name="show_zoom_toolbar"          append="show_merge" />
	     <Action name="show_status_bar"            append="show_merge" />
	</Menu>
         <Menu name="rulers" append="show_merge"><text>&amp;Rulers</text>
	     <Action name="show_rulers"                append="show_merge" />
	     <Action name="show_chord_name_ruler"      append="show_merge" />
	     <Action name="show_tempo_ruler"           append="show_merge" />
	 </Menu>
         <Separator append="show_merge" />
	     <Action name="show_transport"             append="show_merge" />
	     <Action name="show_inst_segment_parameters"       append="show_merge" />
         <Separator append="show_merge" />
             <Action name="show_previews"              append="show_merge" />
             <Action name="show_segment_labels"        append="show_merge" />
             <Action name="show_tracklabels"           append="show_merge" />
         <!-- Action name="toggle_all"                 append="show_merge" /> -->
  </Menu>
  <Menu name="help" append="about_merge"><text>&amp;Help</text>
    <Action name="tutorial"/>
    <Action name="guidelines"/>
  </Menu>

</MenuBar>

<ToolBar name="Tools Toolbar">
     <text>Tools Toolbar</text>
        <Action name="select"/>
        <Action name="draw"/>
        <Action name="erase"/>
        <Action name="move"/>
        <Action name="resize"/>
        <Action name="split"/>
        <!-- <Action name="join"/> -->
</ToolBar>

<ToolBar name="Tracks Toolbar">
    <text>Tracks Toolbar</text>
        <Action name="add_track"/>
        <Action name="delete_track"/>
        <Action name="move_track_up"/>
        <Action name="move_track_down"/>
	<Separator/>
        <Action name="mute_all_tracks"/>
        <Action name="unmute_all_tracks"/>
<!--	<Separator/>
	<Action name="show_previews"/>
	<Action name="show_segment_labels"/>
	<Action name="show_tracklabels"/>  -->
</ToolBar>

<ToolBar name="Editors Toolbar">
     <text>Editors Toolbar</text>
        <Action name="edit_matrix"/>
        <Action name="edit_percussion_matrix"/>
        <Action name="edit_notation"/>
        <Action name="edit_event_list"/>
	<Action name="audio_manager"/>
	<Separator/>
        <Action name="quantize_selection"/>
	<Separator/>
	<Action name="manage_devices"/>
	<Action name="manage_synths"/>
        <Action name="midi_mixer"/>
        <Action name="audio_mixer"/>
</ToolBar>

<ToolBar name="Transport Toolbar">
  <text>Transport Toolbar</text>
  <Action name="rewindtobeginning" />
  <Action name="rewind" />
  <Action name="play" />
  <Action name="fast_forward" />
  <Action name="fastforwardtoend" />
  <Action name="stop" />
  <Action name="record" />
  <Action name="toggle_tracking"/>
  <Action name="panic"/>
</ToolBar>

<ToolBar name="Zoom Toolbar">
  <text>Zoom Toolbar</text>
</ToolBar>

<Menu noMerge="1" name="segment_tool_menu">
   <Action name="edit_default"/>
   <Separator/>
   <Action name="edit_matrix"/>
   <Action name="edit_percussion_matrix"/>
   <Action name="edit_notation"/>
   <Action name="edit_event_list"/>
   <Separator/>
   <Action name="edit_undo"/>
   <Action name="edit_redo"/>
   <Separator/>
   <Action name="edit_cut"/>
   <Action name="edit_copy"/>
   <Action name="edit_paste"/>
   <Separator/>
   <Action name="delete"/>
   <Action name="join_segments"/>
   <Separator/>
   <Action name="quantize_selection"/>
   <Action name="repeat_quantize"/>
   <Action name="relabel_segment"/>
   <Action name="transpose"/>
   <Separator/>
   <Action name="select"/>
   <Action name="move"/>
   <Action name="draw"/>
   <Action name="erase"/>
   <Action name="resize"/>
   <Action name="split"/>
   <!-- <Action name="join"/> -->
</Menu>

<State name="new_file">
        <disable>
                <Action name="file_revert"/>
        </disable>
        <enable>
                <Action name="file_close"/>
                <Action name="file_print"/>
                <Action name="file_print_preview"/>
        </enable>
</State>

<State name="new_file_modified">
        <enable>
        </enable>
</State>

<State name="saved_file_modified">
        <enable>
                <Action name="file_revert"/>
        </enable>
</State>

<State name="have_project_packager">
	<enable>
		<Action name="file_import_project"/>
		<Action name="file_export_project"/>
	</enable>
</State>

<State name="have_lilypondview">
	<enable>
		<Action name="file_preview_lilypond"/>
	</enable>
</State>

<State name="have_segments">
        <enable>
                <Action name="move"/>
                <Action name="erase"/>
                <Action name="resize"/>
                <Action name="split"/>
                <!-- <Action name="join"/> -->
                <Action name="file_print"/>
                <Action name="file_print_preview"/>
        </enable>
</State>

<State name="have_selection">
     <enable>
        <Action name="edit_cut"/>
        <Action name="edit_copy"/>
        <Action name="delete"/>
        <Action name="edit_default"/>
        <Action name="edit_matrix"/>
        <Action name="edit_percussion_matrix"/>
        <Action name="edit_notation"/>
        <Action name="edit_event_list"/>
        <Action name="quantize_selection"/>
        <Action name="repeat_quantize"/>
<!--    <Action name="harmonize_selection"/> -->
        <Action name="rescale"/>
        <Action name="jog_left"/>
        <Action name="jog_right"/>
        <Action name="auto_split"/>
        <Action name="split_by_pitch"/>
        <Action name="split_by_recording"/>
        <Action name="split_at_time"/>
        <Action name="join_segments"/>
        <Action name="relabel_segment"/>
        <Action name="transpose"/>
        <Action name="set_segment_start"/>
        <Action name="set_segment_duration"/>
        <Action name="collapse"/>
        <Action name="set_tempo_to_segment_length"/>
        <Action name="repeats_to_real_copies"/>
        <Action name="groove_quantize"/>
     </enable>
</State>

<State name="have_clipboard">
     <enable>
        <Action name="edit_paste"/>
        <Action name="paste_range"/>
     </enable>
</State>

<State name="audio_segment_selected">
     <enable>
        <Action name="set_tempo_to_segment_length"/>
     </enable>
      <disable>
        <Action name="edit_notation"/>
        <Action name="edit_matrix"/>
        <Action name="edit_percussion_matrix"/>
        <Action name="edit_event_list"/>
        <Action name="quantize_selection"/>
        <Action name="repeat_quantize"/>
<!--        <Action name="rescale"/> -->
        <Action name="split_by_pitch"/>
        <Action name="split_by_recording"/>
        <Action name="join_segments"/>
        <Action name="collapse"/>
        <Action name="repeats_to_real_copies"/>
        <Action name="groove_quantize"/>
     </disable>
</State>

<State name="got_midi_devices">
     <enable>
       <Action name="modify_banks"/>
       <Action name="remap_instruments"/>
       <Action name="midi_mixer"/>
     </enable>
</State>

<State name="got_audio">
     <enable>
	<Action name="audio_manager"/>
	<Action name="audio_mixer"/>
     </enable>
</State>

<State name="sequencer_running">
     <enable>
       <Action name="rewindtobeginning" />
       <Action name="rewind" />
       <Action name="play" />
       <Action name="fast_forward" />
       <Action name="fastforwardtoend" />
       <Action name="stop" />
       <Action name="record" />
       <Action name="recordtoggle" />
     </enable>
</State>

<State name="not_playing">
     <enable>
     </enable>
</State>

<State name="parametersbox_closed">
    <enable>
         <Action name="show_inst_segment_parameters" />
    </enable>
</State>

<State name="have_range">
    <enable>
	 <Action name="cut_range"/>
	 <Action name="copy_range"/>
    </enable>
</State>

</kpartgui>
