/*
    Rosegarden
    A MIDI and audio sequencer and musical notation editor.

    This program is Copyright 2000-2008
        Guillaume Laurent   <glaurent@telegraph-road.org>,
        Chris Cannam        <cannam@all-day-breakfast.com>,
        Richard Bown        <richard.bown@ferventsoftware.com>

    The moral rights of Guillaume Laurent, Chris Cannam, and Richard
    Bown to claim authorship of this work have been asserted.

    Other copyrights also apply to some parts of this work.  Please
    see the AUTHORS file and individual file headers for details.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/

#ifndef _RG_MIDIPROGRAMSEDITOR_H_
#define _RG_MIDIPROGRAMSEDITOR_H_

#include "base/MidiProgram.h"
#include "NameSetEditor.h"


class TQWidget;
class TQString;
class TQSpinBox;
class TQListViewItem;
class TQCheckBox;
class BankList;


namespace Rosegarden
{

class MidiProgram;
class MidiDevice;
class BankEditorDialog;


class MidiProgramsEditor : public NameSetEditor
{
    TQ_OBJECT
  
public:
    MidiProgramsEditor(BankEditorDialog *bankEditor,
                       TQWidget *parent,
                       const char *name = 0);

    void clearAll();
    void populate(TQListViewItem*);
    void reset();

public slots:

    // Check that any new MSB/LSB combination is unique for this device
    //
    void slotNewMSB(int value);
    void slotNewLSB(int value);
    void slotNewPercussion(); // gets value from checkbox

    virtual void slotNameChanged(const TQString &);
    virtual void slotEntryButtonPressed();
    void slotEntryMenuItemSelected(int);

protected:

    MidiBank* getCurrentBank();

    int ensureUniqueMSB(int msb, bool ascending);
    int ensureUniqueLSB(int lsb, bool ascending);

    // Does the banklist contain this combination already?
    //
    bool banklistContains(const MidiBank &);

    ProgramList getBankSubset(const MidiBank &);

    /// Set the currently loaded programs to new MSB and LSB
    void modifyCurrentPrograms(const MidiBank &oldBank,
                               const MidiBank &newBank);
    
    // Get a program (pointer into program list) for modification
    //
    MidiProgram* getProgram(const MidiBank &bank, int program);

    void setBankName(const TQString& s);

    virtual TQWidget *makeAdditionalWidget(TQWidget *parent);

    void blockAllSignals(bool block);

    //--------------- Data members ---------------------------------
    TQCheckBox                *m_percussion;
    TQSpinBox                 *m_msb;
    TQSpinBox                 *m_lsb;

    MidiDevice   *m_device;

    MidiBank     *m_currentBank;
    BankList     &m_bankList;
    ProgramList  &m_programList;

    MidiBank      m_oldBank;

    unsigned int              m_currentMenuProgram;
};


}

#endif
