/*
    Rosegarden
    A MIDI and audio sequencer and musical notation editor.

    This program is Copyright 2000-2008
        Guillaume Laurent   <glaurent@telegraph-road.org>,
        Chris Cannam        <cannam@all-day-breakfast.com>,
        Richard Bown        <richard.bown@ferventsoftware.com>

    The moral rights of Guillaume Laurent, Chris Cannam, and Richard
    Bown to claim authorship of this work have been asserted.

    Other copyrights also apply to some parts of this work.  Please
    see the AUTHORS file and individual file headers for details.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/

#ifndef _RG_PROPERTYCONTROLRULER_H_
#define _RG_PROPERTYCONTROLRULER_H_

#include "base/PropertyName.h"
#include "base/Staff.h"
#include "ControlRuler.h"
#include <tqstring.h>
#include "base/Event.h"


class TQWidget;
class TQMouseEvent;
class TQContextMenuEvent;
class TQCanvasLine;
class TQCanvas;


namespace Rosegarden
{

class ViewElement;
class Staff;
class Segment;
class RulerScale;
class EditViewBase;


/**
 * PropertyControlRuler : edit a property on events on a staff (only
 * events with a ViewElement attached, mostly notes)
 */
class PropertyControlRuler : public ControlRuler, public StaffObserver
{
public:
    PropertyControlRuler(PropertyName propertyName,
                         Staff*,
                         RulerScale*,
                         EditViewBase* parentView,
                         TQCanvas*,
                         TQWidget* parent=0, const char* name=0, WFlags f=0);

    virtual ~PropertyControlRuler();

    virtual TQString getName();

    const PropertyName& getPropertyName()     { return m_propertyName; }

    // Allow something external to reset the selection of Events
    // that this ruler is displaying
    //
    void setStaff(Staff *);

    // StaffObserver interface
    virtual void elementAdded(const Staff *, ViewElement*);
    virtual void elementRemoved(const Staff *, ViewElement*);
    virtual void staffDeleted(const Staff *);
    virtual void startPropertyLine();
    virtual void selectAllProperties();

    /// SegmentObserver interface
    virtual void endMarkerTimeChanged(const Segment *, bool shorten);

protected:

    virtual void contentsMousePressEvent(TQMouseEvent*);
    virtual void contentsMouseReleaseEvent(TQMouseEvent*);
    virtual void contentsMouseMoveEvent(TQMouseEvent*);
    virtual void contentsContextMenuEvent(TQContextMenuEvent*);

    void drawPropertyLine(timeT startTime,
                          timeT endTime,
                          int startValue,
                          int endValue);

    virtual void init();
    virtual void drawBackground();
    virtual void computeStaffOffset();

    //--------------- Data members ---------------------------------

    PropertyName       m_propertyName;
    Staff*             m_staff;

    TQCanvasLine                   *m_propertyLine;
    
    bool                           m_propertyLineShowing;
    int                            m_propertyLineX;
    int                            m_propertyLineY;
};



}

#endif
