<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="Rosegarden::Mixer" version="1230">
  
<MenuBar>

  <Menu name="Transport">
        <text>T&amp;ransport</text>
  <Action name="play" />
  <Action name="stop" />
  <Action name="playback_pointer_back_bar" />
  <Action name="playback_pointer_forward_bar" />
  <Action name="playback_pointer_start" />
  <Action name="playback_pointer_end" />
  <Action name="record" />
  <Action name="panic" />
  </Menu>

<Menu name="settings"><text>&amp;Settings</text>

  <Action name="show_midi" append="show_merge" />
  <Action name="show_audio_faders" append="show_merge" />
  <Action name="show_synth_faders" append="show_merge" />
  <Action name="show_audio_submasters" append="show_merge" />
  <Action name="show_plugin_buttons" append="show_merge" />

  <Separator append="show_merge"/>

  <Action name="show_unassigned_faders" append="show_merge" />

  <Menu name="Inputs">
    <text>Number of Stereo &amp;Inputs</text>
    <Action name="inputs_1"/>
    <Action name="inputs_2"/>
    <Action name="inputs_4"/>
    <Action name="inputs_8"/>
    <Action name="inputs_16"/>
  </Menu>

  <Menu name="Submasters">
    <text>&amp;Number of Submasters</text>
    <Action name="submasters_0"/>
    <Action name="submasters_2"/>
    <Action name="submasters_4"/>
    <Action name="submasters_8"/>
  </Menu>

</Menu> 

</MenuBar>

<ToolBar name="Transport Toolbar">
  <text>Transport Toolbar</text>
  <Action name="playback_pointer_start" />
  <Action name="playback_pointer_back_bar" />
  <Action name="play" />
  <Action name="playback_pointer_forward_bar" />
  <Action name="playback_pointer_end" />
  <Action name="stop" />
  <Action name="record" />
  <Action name="panic" />
</ToolBar>

</kpartgui>
 
