<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="Rosegarden::NotationTool" version="1230">

<Menu noMerge="1" name="TextInserter">
  <Action name="select" />
  <Action name="notes" />
  <Action name="erase" />
</Menu>

</kpartgui>
