<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="TriggerManager" version="1230">

<MenuBar>

  <Menu name="file">
    <Action name="file_close_discard"/>
  </Menu>

  <Menu name="edit">
    <Action name="paste_to_trigger_segment"/>
  </Menu>

  <Menu name="view"><text>&amp;View</text>
    <Action name="time_musical"/>
    <Action name="time_real"/>
    <Action name="time_raw"/>
  /Menu>

</MenuBar>

<ToolBar name="mainToolBar">
   <Action name="file_close"/>
</ToolBar>

<ToolBar name="Time Toolbar">
  <text>Time Toolbar</text>
  <Action name="time_musical"/>
  <Action name="time_real"/>
  <Action name="time_raw"/>
</ToolBar>

<State name="have_clipboard_single_segment">
     <enable>
        <Action name="paste_to_trigger_segment"/>
     </enable>
</State>

</kpartgui>
