/*
    Rosegarden
    A MIDI and audio sequencer and musical notation editor.

    This program is Copyright 2000-2008
        Guillaume Laurent   <glaurent@telegraph-road.org>,
        Chris Cannam        <cannam@all-day-breakfast.com>,
        Richard Bown        <richard.bown@ferventsoftware.com>

    The moral rights of Guillaume Laurent, Chris Cannam, and Richard
    Bown to claim authorship of this work have been asserted.

    Other copyrights also apply to some parts of this work.  Please
    see the AUTHORS file and individual file headers for details.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/

#ifndef _RG_TQDEFERSCROLLVIEW_H_
#define _RG_TQDEFERSCROLLVIEW_H_

#include <tqscrollview.h>


class TQWidget;
class TQWheelEvent;


namespace Rosegarden
{



/**
 * A TQScrollView which defers vertical scrolling (through mouse wheel)
 * elsewhere, typically another TQScrollView, so that both can be kept
 * in sync. The master scrollview will connect its vertical scrollbar
 * to the slave view so the scrollbar will act on both views.
 *
 * The slave scrollview will defer its scrolling to the master by
 * having the gotWheelEvent() signal connected to a slot in the master
 * scrollview, which will simply process the wheel event as if it had
 * received it itself.
 *
 * @see TrackEditor
 * @see SegmentCanvas
 * @see TrackEditor::m_trackButtonScroll
 */
class QDeferScrollView : public TQScrollView
{
    TQ_OBJECT
  
public:
    QDeferScrollView(TQWidget* parent=0, const char *name=0, WFlags f=0);

    void setBottomMargin(int);

signals:
    void gotWheelEvent(TQWheelEvent*);

protected:
    virtual void contentsWheelEvent(TQWheelEvent*);
    
};


}

#endif
