<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="Rosegarden::MatrixTool" version="1230">

<Menu name="MatrixResizer">
  <Action name="select" />
  <Action name="move"/>
  <Action name="draw"/>
  <Action name="erase"/>
<!-- <Separator/> -->
<!-- Defunct: <Action name="halve_durations"/> -->
<!-- Defunct: <Action name="double_durations"/> -->
</Menu>

</kpartgui>
