<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="Rosegarden::NotationTool" version="1230">

<Menu name="NotationEraser">
  <Action name="select" />
  <Action name="insert" />
  <Separator/>
  <Action name="toggle_rest_collapse" />
</Menu>

</kpartgui>
