#ifndef STABLEHEADERS_H_
#define STABLEHEADERS_H_

// Standard C++ library
#include <algorithm>
#include <cassert>
#include <cctype>
#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>
#include <deque>
#include <exception>
#include <fstream>
#include <iomanip>
#include <ios>
#include <iostream>
#include <iterator>
#include <list>
#include <map>
#include <set>
#include <sstream>
#include <stack>
#include <string>
#include <utility>
#include <vector>

// QT3 headers

// Common headers, or used by sources generated by moc
#include <tqmap.h> 
#include <tqglobal.h>
#include <private/tqucomextra_p.h> 
#include <tqmetaobject.h>
#include <tqobjectdefs.h>
#include <tqsignalslotimp.h>
#include <tqstyle.h>

// Headers used by Rosegarden or KDE3
#include <tqaccel.h>
#include <tqapplication.h>
#include <tqbitmap.h>
#include <tqbrush.h>
#include <tqbuffer.h>
#include <tqbuttongroup.h>
#include <tqbutton.h>
#include <tqcanvas.h>
#include <tqcheckbox.h>
#include <tqcolor.h>
#include <tqcombobox.h>
#include <tqcstring.h>
#include <tqcursor.h>
#include <tqdatastream.h>
#include <tqdatetime.h>
#include <tqdialog.h>
#include <tqdict.h>
#include <tqdir.h>
#include <tqdom.h>
#include <tqdragobject.h>
#include <tqdrawutil.h>
#include <tqevent.h>
#include <tqeventloop.h>
#include <tqfile.h>
#include <tqfileinfo.h>
#include <tqfont.h>
#include <tqfontinfo.h>
#include <tqfontmetrics.h>
#include <tqframe.h>
#include <tqgarray.h>
#include <tqgrid.h>
#include <tqgroupbox.h>
#include <tqguardedptr.h>
#include <tqhbox.h>
#include <tqheader.h>
#include <tqiconset.h>
#include <tqimage.h>
#include <tqinputdialog.h>
#include <tqlabel.h>
#include <tqlayout.h>
#include <tqlineedit.h>
#include <tqlistbox.h>
#include <tqlistview.h>
#include <tqmemarray.h>
#include <tqmutex.h>
#include <tqobject.h>
#include <tqobjectlist.h>
#include <tqpaintdevicemetrics.h>
#include <tqpainter.h>
#include <tqpalette.h>
#include <tqpen.h>
#include <tqpixmap.h>
#include <tqpointarray.h>
#include <tqpoint.h>
#include <tqpopupmenu.h>
#include <tqprinter.h>
#include <tqprogressdialog.h>
#include <tqptrdict.h>
#include <tqptrlist.h>
#include <tqpushbutton.h>
#include <tqradiobutton.h>
#include <tqrect.h>
#include <tqregexp.h>
#include <tqregion.h>
#include <tqscrollbar.h>
#include <tqscrollview.h>
#include <tqsessionmanager.h>
#include <tqsignalmapper.h>
#include <tqsize.h>
#include <tqsizepolicy.h>
#include <tqslider.h>
#include <tqsocketnotifier.h>
#include <tqspinbox.h>
#include <tqsplitter.h>
#include <tqstring.h>
#include <tqstringlist.h>
#include <tqstrlist.h>
#include <tqtable.h>
#include <tqtabwidget.h>
#include <tqtextcodec.h>
#include <tqtextedit.h>
#include <tqtextstream.h>
#include <tqthread.h>
#include <tqtimer.h>
#include <tqtoolbutton.h>
#include <tqtooltip.h>
#include <tqvalidator.h>
#include <tqvaluelist.h>
#include <tqvaluevector.h>
#include <tqvariant.h>
#include <tqvbox.h>
#include <tqvgroupbox.h>
#include <tqwhatsthis.h>
#include <tqwidget.h>
#include <tqwidgetstack.h>
#include <tqwmatrix.h>
#include <tqxml.h>

// KDE3 headers
#include <dcopclient.h>
#include <dcopobject.h>
#include <dcopref.h>
#include <tdeaboutdata.h>
#include <tdeaccel.h>
#include <tdeactioncollection.h>
#include <tdeaction.h>
#include <tdeapplication.h>
#include <karrowbutton.h>
#include <tdecmdlineargs.h>
#include <kcolordialog.h>
#include <kcombobox.h>
#include <kcommand.h>
#include <kcompletion.h>
#include <tdeconfig.h>
#include <kcursor.h>
#include <kdcopactionproxy.h>
#include <kdebug.h>
#include <kdialogbase.h>
#include <kdialog.h>
#include <kdiskfreesp.h>
#include <kdockwidget.h>
#include <kedittoolbar.h>
#include <tdefiledialog.h>
#include <tdefile.h>
#include <kfilterdev.h>
#include <tdefontrequester.h>
#include <tdeglobal.h>
#include <tdeglobalsettings.h>
#include <kiconloader.h>
#include <kinputdialog.h>
#include <tdeio/netaccess.h>
#include <kkeydialog.h>
#include <kled.h>
#include <klineeditdlg.h>
#include <klineedit.h>
#include <tdelistview.h>
#include <tdelocale.h>
#include <tdemainwindow.h>
#include <tdemessagebox.h>
#include <kmimetype.h>
#include <kpixmapeffect.h>
#include <tdepopupmenu.h>
#include <kprinter.h>
#include <kprocess.h>
#include <kpushbutton.h>
#include <ksqueezedtextlabel.h>
#include <kstandarddirs.h>
#include <kstatusbar.h>
#include <tdestdaccel.h>
#include <kstdaction.h>
#include <kstandarddirs.h>
#include <ktabwidget.h>
#include <tdetempfile.h>
#include <ktip.h>
#include <tdetoolbar.h>
#include <kuniqueapplication.h>
#include <kurl.h>
#include <kxmlguiclient.h>
#include <kxmlguifactory.h>

#define private protected // fugly
#include <kprogress.h>
#undef private

#endif /*STABLEHEADERS_H_*/
