/*
    Rosegarden
    A MIDI and audio sequencer and musical notation editor.

    This program is Copyright 2000-2008
        Guillaume Laurent   <glaurent@telegraph-road.org>,
        Chris Cannam        <cannam@all-day-breakfast.com>,
        Richard Bown        <richard.bown@ferventsoftware.com>

    The moral rights of Guillaume Laurent, Chris Cannam, and Richard
    Bown to claim authorship of this work have been asserted.

    Other copyrights also apply to some parts of this work.  Please
    see the AUTHORS file and individual file headers for details.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/

#ifndef _RG_COMPOSITIONRECT_H_
#define _RG_COMPOSITIONRECT_H_

#include <tqbrush.h>
#include <tqcolor.h>
#include <tqpen.h>
#include <tqrect.h>
#include <tqstring.h>
#include <tqvaluevector.h>


class TQSize;
class TQPoint;


namespace Rosegarden
{

class CompositionRect : public TQRect
{
public:
    typedef TQValueVector<int> repeatmarks;

    friend bool operator<(const CompositionRect&, const CompositionRect&);

    CompositionRect() : TQRect(), m_selected(false),
                        m_needUpdate(false), m_brush(DefaultBrushColor), m_pen(DefaultPenColor) {};
    CompositionRect(const TQRect& r) : TQRect(r), m_resized(false), m_selected(false),
                                      m_needUpdate(false), m_brush(DefaultBrushColor), m_pen(DefaultPenColor), m_z(0) {};
    CompositionRect(const TQPoint & topLeft, const TQPoint & bottomRight)
        : TQRect(topLeft, bottomRight), m_resized(false), m_selected(false),
          m_needUpdate(false), m_brush(DefaultBrushColor), m_pen(DefaultPenColor), m_z(0) {};
    CompositionRect(const TQPoint & topLeft, const TQSize & size)
        : TQRect(topLeft, size), m_resized(false), m_selected(false),
          m_needUpdate(false), m_brush(DefaultBrushColor), m_pen(DefaultPenColor), m_z(0) {};
    CompositionRect(int left, int top, int width, int height)
        : TQRect(left, top, width, height), m_resized(false), m_selected(false),
          m_needUpdate(false), m_brush(DefaultBrushColor), m_pen(DefaultPenColor), m_z(0) {};

    void setResized(bool s)       { m_resized = s; }
    bool isResized() const        { return m_resized; }
    void setSelected(bool s)      { m_selected = s; }
    bool isSelected() const       { return m_selected; }
    bool needsFullUpdate() const  { return m_needUpdate; }
    void setNeedsFullUpdate(bool s) { m_needUpdate = s; }

    void setZ(int z) { m_z = z; }
    int z() const { return m_z; }
    
    // brush, pen draw info
    void setBrush(TQBrush b)       { m_brush = b; }
    TQBrush getBrush() const       { return m_brush; }
    void setPen(TQPen b)           { m_pen = b; }
    TQPen getPen() const           { return m_pen; }

    // repeating segments
    void                setRepeatMarks(const repeatmarks& rm) { m_repeatMarks = rm; }
    const repeatmarks&  getRepeatMarks() const                { return m_repeatMarks; }
    bool                isRepeating() const                   { return m_repeatMarks.size() > 0; }
    int                 getBaseWidth() const                  { return m_baseWidth; }
    void                setBaseWidth(int bw)                  { m_baseWidth = bw; }
    TQString             getLabel() const                      { return m_label; }
    void                setLabel(TQString l)                   { m_label = l; }

    static const TQColor DefaultPenColor;
    static const TQColor DefaultBrushColor;

protected:
    bool        m_resized;
    bool        m_selected;
    bool        m_needUpdate;
    TQBrush      m_brush;
    TQPen        m_pen;
    repeatmarks m_repeatMarks;
    int         m_baseWidth;
    TQString     m_label;
    int         m_z;
};


}

#endif
