/*
    Rosegarden
    A MIDI and audio sequencer and musical notation editor.
 
    This program is Copyright 2000-2008
        Guillaume Laurent   <glaurent@telegraph-road.org>,
        Chris Cannam        <cannam@all-day-breakfast.com>,
        Richard Bown        <richard.bown@ferventsoftware.com>
 
    The moral rights of Guillaume Laurent, Chris Cannam, and Richard
    Bown to claim authorship of this work have been asserted.

    This file taken from KMix
    Copyright (C) 2000 Stefan Schimanski <1Stein@gmx.de>.
 
    Other copyrights also apply to some parts of this work.  Please
    see the AUTHORS file and individual file headers for details.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/


#include "KLedButton.h"

#include <kled.h>
#include <tqcolor.h>
#include <tqwidget.h>


namespace Rosegarden
{

KLedButton::KLedButton(const TQColor &col, TQWidget *parent, const char *name)
        : KLed( col, parent, name )
{}

KLedButton::KLedButton(const TQColor& col, KLed::State st, KLed::Look look,
                       KLed::Shape shape, TQWidget *parent, const char *name)
        : KLed( col, st, look, shape, parent, name )
{}

KLedButton::~KLedButton()
{}

void KLedButton::mousePressEvent( TQMouseEvent *e )
{
    if (e->button() == TQt::LeftButton) {
        toggle();
        emit stateChanged( state() );
    }
}

}
#include "KLedButton.moc"
