/*
    Rosegarden
    A sequencer and musical notation editor.

    This program is Copyright 2000-2008
        Guillaume Laurent   <glaurent@telegraph-road.org>,
        Chris Cannam        <cannam@all-day-breakfast.com>,
        Richard Bown        <bownie@bownie.com>

    The moral right of the authors to claim authorship of this work
    has been asserted.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.  See the file
    COPYING included with this distribution for more information.
*/

#ifndef _ROSE_STRINGS_H_
#define _ROSE_STRINGS_H_

#include <string>
#include <tqstring.h>
#include "PropertyName.h"
#include "Exception.h"

extern TQString strtoqstr(const std::string &);
extern TQString strtoqstr(const Rosegarden::PropertyName &);
extern std::string qstrtostr(const TQString &);
extern double strtodouble(const std::string &);
extern double qstrtodouble(const TQString &);

class TQTextCodec;
extern std::string convertFromCodec(std::string, TQTextCodec *);

#endif
