<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="Rosegarden::Matrix" version="1500">
  
<MenuBar>

<Menu name="file" append="new_merge">
</Menu>

<Menu name="edit">
  <Action name="delete"/>
  <Action name="select_all"/>
  <Action name="clear_selection"/>
  <Action name="filter_selection"/>
</Menu>


<Menu name="View">
  <text>&amp;View</text>
  <Menu name="Grid">
    <text>&amp;Grid</text>
    <Action name="snap_none"/>
    <Action name="snap_unit"/>
    <Action name="snap_64"/>
    <Action name="snap_48"/>
    <Action name="snap_32"/>
    <Action name="snap_24"/>
    <Action name="snap_16"/>
    <Action name="snap_12"/>
    <Action name="snap_8"/>
    <Action name="snap_6"/>
    <Action name="snap_4"/>
    <Action name="snap_2"/>
    <Action name="snap_beat"/>
    <Action name="snap_bar"/>
  </Menu>
  <Separator/>
    <Action name="show_velocity_control_ruler"/>
    <Action name="show_controller_events_ruler"/>
<!--    <Action name="add_control_ruler"/> -->
    <Menu name="add_control_ruler"><text>Add Event Ruler</text></Menu>
</Menu> 

<Menu name="document">
	<text>&amp;Composition</text>
	<Action name="add_tempo"/>
	<Action name="add_time_signature"/>
</Menu>
<Menu name="segments">
	<text>&amp;Segment</text>
	<Menu name="open-with">
	        <text>Edit &amp;With</text>
		<Action name="open_in_event_list"/>
	        <Action name="open_in_notation"/>
	</Menu>
        <Separator/>	
	<Action name="set_segment_start"/>
	<Action name="set_segment_duration"/>
</Menu>
<Menu name="Transforms">
  <text>Ad&amp;just</text>
   <Menu name="quantize_notes">
	 <text>&amp;Quantize</text>
   	 <Action name="quantize"/>
	 <Action name="repeat_quantize"/>
	 <Action name="legatoize"/>
  </Menu>
  <Action name="collapse_notes"/>
  <Separator/>
  <Action name="jog_left"/>
  <Action name="jog_right"/>
  <Separator/>
  <Action name="velocity_up"/>
  <Action name="velocity_down"/>
  <Action name="set_to_current_velocity"/>
  <Action name="set_velocities"/>
  <Separator/>
  <Menu name="Rescale">
    <text>Rescale</text>
    <Action name="halve_durations"/>
    <Action name="double_durations"/>
    <Separator/>
    <Action name="rescale"/>
  </Menu>
  <Menu name="Transpose">
    <text>Trans&amp;pose</text>
    <Action name="transpose_up"/>
    <Action name="transpose_down"/>
    <Separator/>
    <Action name="transpose_up_octave"/>
    <Action name="transpose_down_octave"/>
    <Separator/>
    <Action name="general_transpose"/>
    <Action name="general_diatonic_transpose"/>
  </Menu>
  <Menu name="Convert">
    <text>Convert</text>
    <Action name="invert"/>
    <Action name="retrograde"/>
    <Action name="retrograde_invert"/>
  </Menu>
  <Separator/>
  <Action name="trigger_segment"/>
  <Action name="remove_trigger"/>
</Menu>


<Menu name="Tools">
  <text>&amp;Tools</text>
	<Action name="select" />
	<Action name="draw" />
	<Action name="erase" />
	<Action name="move" />
        <Action name="resize" />
	<Separator/>
    <Menu name="Move">
        <text>Local &amp;Cursor</text>
        <Action name="cursor_back"/>
        <Action name="cursor_forward"/>
        <Action name="cursor_back_bar"/>
        <Action name="cursor_forward_bar"/>
        <Action name="cursor_start"/>
        <Action name="cursor_end"/>
        <Separator/>
        <Action name="extend_selection_backward"/>
        <Action name="extend_selection_forward"/>
        <Action name="preview_selection"/>
        <Action name="clear_loop"/>
        <Separator/>
        <Action name="cursor_to_playback_pointer"/>
        <Action name="playback_pointer_to_cursor"/>
    </Menu>

    <Menu name="Transport">
    <text>T&amp;ransport</text>
        <Action name="play"/> 
        <Action name="stop"/> 
        <Action name="playback_pointer_back_bar"/>
        <Action name="playback_pointer_forward_bar"/>
        <Action name="playback_pointer_start"/>
        <Action name="playback_pointer_end"/>
        <Action name="playback_pointer_to_cursor" />
        <Action name="cursor_to_playback_pointer" />
        <Action name="toggle_solo"/>
        <Action name="toggle_tracking"/>
        <Action name="panic"/>
    </Menu>

  <Separator/>
  <Action name="insert_note_actionmenu"/>
  <Action name="chord_mode"/>

  <Separator/>
  <Action name="toggle_step_by_step"/>
</Menu>

<Menu  name="settings" append="show_merge"><text>&amp;Settings</text>
  <Menu name="toolbars" append="show_merge"><text>&amp;Toolbars</text>
    <Action name="options_show_toolbar"      append="show_merge" />
    <Action name="show_tools_toolbar"        append="show_merge" />
    <Action name="options_show_statusbar"    append="show_merge" />
  </Menu>
  <Menu name="rulers" append="show_merge"><text>&amp;Rulers</text>
    <Action name="show_chords_ruler"         append="show_merge" />
    <Action name="show_tempo_ruler"          append="show_merge" />
  </Menu>
  <Separator                                 append="show_merge" />
    <Action name="show_inst_parameters"      append="show_merge" />
</Menu> 


</MenuBar>

<ToolBar name="Tools Toolbar">
  <text>Tools Toolbar</text>

  <Action name="select" />
  <Action name="draw" />
  <Action name="erase" />
  <Action name="move" />
  <Action name="resize" />
  <Separator/>
  <Action name="toggle_step_by_step"/>
  <Action name="filter_selection"/>
  <Action name="quantize"/>
</ToolBar>

<ToolBar name="Transport Toolbar">
  <text>Transport Toolbar</text>

  <Action name="playback_pointer_start" />
  <Action name="playback_pointer_back_bar" />
  <Action name="play" />
  <Action name="playback_pointer_forward_bar" />
  <Action name="playback_pointer_end" />
  <Action name="stop" />
  <Action name="playback_pointer_to_cursor" />
  <Action name="cursor_to_playback_pointer" />
  <Action name="toggle_solo"/>
  <Action name="toggle_tracking"/>
  <Action name="panic"/>
</ToolBar>

<ToolBar name="Actions Toolbar">
  <text>Actions Toolbar</text>
</ToolBar>

<ToolBar name="Zoom Toolbar">
  <text>Zoom Toolbar</text>
</ToolBar>

<Menu noMerge="1" name="controller_events_ruler_menu">
   <Text>Control Ruler actions</Text>
   <Action name="insert_control_ruler_item"/>
   <Action name="erase_control_ruler_item"/>
   <Action name="clear_control_ruler_item"/>
   <Separator/>
   <Action name="start_control_line_item"/>
   <Separator/>
   <Action name="flip_control_events_forward"/>
   <Action name="flip_control_events_back"/>
</Menu>

<Menu noMerge="1" name="property_ruler_menu">
   <Text>Property Ruler actions</Text>
   <Action name="draw_property_line"/>
   <Action name="select_all_properties"/>
   <Separator/>
   <Action name="flip_control_events_forward"/>
   <Action name="flip_control_events_back"/>
</Menu>


<State name="have_selection">
  <enable>
    <Action name="edit_cut"/>
    <Action name="edit_copy"/>
    <Action name="quantize"/>
    <Action name="collapse_notes"/>
    <Action name="legatoize"/>
    <Action name="rescale"/>
    <Action name="double_durations"/>
    <Action name="halve_durations"/>
    <Action name="velocity_up"/>
    <Action name="velocity_down"/>
    <Action name="jog_left"/>
    <Action name="jog_right"/>
    <Action name="set_to_current_velocity"/>
    <Action name="set_velocities"/>
    <Action name="trigger_segment"/>
    <Action name="remove_trigger"/>
    <Action name="transpose_up"/>
    <Action name="transpose_down"/>
    <Action name="transpose_up_octave"/>
    <Action name="transpose_down_octave"/>
    <Action name="general_transpose"/>
    <Action name="general_diatonic_transpose"/>
    <Action name="invert"/>
    <Action name="retrograde"/>
    <Action name="retrograde_invert"/>
    <Action name="preview_selection"/>
    <Action name="clear_selection"/>
    <Action name="filter_selection"/>
  </enable>
</State>

<State name="have_clipboard_single_segment">
  <enable>
    <Action name="edit_paste"/>
  </enable>
</State>

<State name="have_note_events_in_segment">
  <enable>
   <Action name="draw_property_line"/>
  </enable>
</State>

<State name="have_control_ruler">
  <enable>
    <Action name="show_control_rulers"/>
  </enable>
</State>

<State name="have_controller_item_selected">
  <enable>
    <Action name="erase_control_ruler_item"/>
  </enable>
  <disable>
   <Action name="insert_control_ruler_item"/>
  </disable>
</State>

<State name="parametersbox_closed">
    <enable>
         <Action name="show_inst_parameters" />
    </enable>
</State>

</kpartgui>
 
